@@ -1,18 +1,16 @@
 // build-fail
 
 // Regression test for #66975
-#![warn(const_err)]
-#![feature(const_panic)]
 #![feature(never_type)]
 
 struct PrintName;
 
 impl PrintName {
     const VOID: ! = panic!();
-    //~^ WARN any use of this value will cause an error
+    //~^ ERROR evaluation of constant value failed
 }
 
 fn main() {
     let _ = PrintName::VOID;
-    //~^ ERROR erroneous constant used
+    //~^ ERROR erroneous constant used [E0080]
 }
